Package: gplots
Title: Various R Programming Tools for Plotting Data
Description: Various R programming tools for plotting data, including:
  - calculating and plotting locally smoothed summary function as
    ('bandplot', 'wapply'),
  - enhanced versions of standard plots ('barplot2', 'boxplot2',
    'heatmap.2', 'smartlegend'),
  - manipulating colors ('col2hex', 'colorpanel', 'redgreen',
    'greenred', 'bluered', 'redblue', 'rich.colors'),
  - calculating and plotting two-dimensional data summaries ('ci2d',
    'hist2d'),
  - enhanced regression diagnostic plots ('lmplot2', 'residplot'),
  - formula-enabled interface to 'stats::lowess' function ('lowess'),
  - displaying textual data in plots ('textplot', 'sinkplot'),
  - plotting dots whose size reflects the relative magnitude of the
    elements ('balloonplot', 'bubbleplot'),
  - plotting "Venn" diagrams ('venn'),
  - displaying Open-Office style plots ('ooplot'),
  - plotting multiple data on same region, with separate axes
    ('overplot'),
  - plotting means and confidence intervals ('plotCI', 'plotmeans'),
  - spacing points in an x-y plot so they don't overlap ('space').
Depends: R (>= 3.0)
Imports: gtools, stats, caTools, KernSmooth, methods
Suggests: grid, MASS, knitr, rmarkdown, r2d2, nlme, dendextend,
        heatmaply, RColorBrewer
LazyData: yes
VignetteBuilder: knitr
Version: 3.3.0
Date: 2025-11-29
Authors@R: c(
    person("Gregory R.", "Warnes", , role = "aut"),
    person("Ben", "Bolker", , role = "aut"),
    person("Lodewijk", "Bonebakker", , role = "aut"),
    person("Robert", "Gentleman", role = "aut"),
    person("Wolfgang", "Huber", role = "aut"),
    person("Andy", "Liaw", role = "aut"),
    person("Thomas", "Lumley", role = "aut"),
    person("Martin", "Maechler", role = "aut"),
    person("Arni", "Magnusson", role = "aut"),
    person("Steffen", "Moeller", role = "aut"),
    person("Marc", "Schwartz", role = "aut"),
    person("Bill", "Venables", role = "aut"),
    person("Tal", "Galili", , "tal.galili@gmail.com", c("aut", "cre"))
    )
License: GPL-2
URL: https://github.com/talgalili/gplots,
        https://talgalili.github.io/gplots/
BugReports: https://github.com/talgalili/gplots/issues
NeedsCompilation: no
Packaged: 2025-11-29 18:15:47 UTC; talgalili
Author: Gregory R. Warnes [aut],
  Ben Bolker [aut],
  Lodewijk Bonebakker [aut],
  Robert Gentleman [aut],
  Wolfgang Huber [aut],
  Andy Liaw [aut],
  Thomas Lumley [aut],
  Martin Maechler [aut],
  Arni Magnusson [aut],
  Steffen Moeller [aut],
  Marc Schwartz [aut],
  Bill Venables [aut],
  Tal Galili [aut, cre]
Maintainer: Tal Galili <tal.galili@gmail.com>
Repository: CRAN
Date/Publication: 2025-11-30 06:10:15 UTC
Built: R 4.6.0; ; 2026-04-26 10:11:01 UTC; unix
