export(
       adjust_hsv,
       angleAxis,
       balloonplot,
       bandplot,
       barplot2,
       bluered,
       boxplot2,
       bubbleplot,
       ci2d,
       col2hex,
       colorpanel,
       greenred,
       heatmap.2,
       hist2d,
       lmplot2,
       lowess,
       ooplot,
       overplot,
       panel.overplot,
       plot.venn,
       plotCI,
       plotLowess,
       plotmeans,
       qqnorm.aov,
       redblue,
       redgreen,
       residplot,
       rich.colors,
       sinkplot,
       space,
       textplot,
       venn,
       wapply,
       reorder.factor
)

importFrom(gtools, invalid)
importFrom(gtools, odd)
importFrom(gtools, mixedsort)

importFrom(stats, reorder)
importFrom(stats, na.omit)
importFrom(stats, nobs)

importFrom(caTools, runsd)
importFrom(caTools, runmean)

importFrom(KernSmooth, bkde2D)
importFrom(KernSmooth, dpik)

importFrom(utils, type.convert)

S3method(balloonplot, default)
S3method(balloonplot, table)

S3method(bandplot, default)
S3method(bandplot, formula)

S3method(barplot2, default)

S3method(bubbleplot, default)
S3method(bubbleplot, formula)

S3method(lowess, default)
S3method(lowess, formula)

S3method(ooplot, default)

S3method(plot, lowess)
S3method(plot, venn)

S3method(print, ci2d)
S3method(print, hist2d)

S3method(qqnorm, aov)

S3method(textplot, character)
S3method(textplot, data.frame)
S3method(textplot, default)
S3method(textplot, matrix)

S3method(reorder, factor)

# imports recommended by R CMD check
importFrom("grDevices", "col2rgb", "contourLines", "dev.flush",
           "dev.hold", "dev.interactive", "heat.colors", "hsv", "rgb",
           "rgb2hsv", "topo.colors")

importFrom("graphics", "abline", "arrows", "axTicks", "axis",
           "barplot", "contour", "filled.contour", "grid", "hist",
           "identify", "image", "layout", "legend", "lines",
           "matlines", "matpoints", "mtext", "panel.smooth", "par",
           "plot", "plot.default", "plot.new", "plot.window", "points",
           "polygon", "rect", "segments", "strheight", "strwidth",
           "text", "title", "xinch", "yinch")

importFrom("stats", "as.dendrogram", "cooks.distance", "density",
           "deviance", "df.residual", "dist", "dnorm", "effects",
           "expand.model.frame", "fitted", "hclust", "is.leaf",
           "lm.influence", "median", "model.frame", "order.dendrogram",
           "ppoints", "predict", "qnorm", "qqline", "qqnorm", "qt",
           "residuals", "sd", "var", "weights")

importFrom("utils", "str")

importFrom("methods", "is")
