[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/gplots)](https://cran.r-project.org/package=gplots)
[![CRAN Monthly](https://cranlogs.r-pkg.org/badges/gplots)](https://cranlogs.r-pkg.org/badges/gplots?color=yellow)
[![CRAN Total](https://cranlogs.r-pkg.org/badges/grand-total/gplots)](https://cranlogs.r-pkg.org/badges/grand-total/gplots?color=yellowgreen)
[![codecov.io](https://codecov.io/github/talgalili/gplots/coverage.svg?branch=master)](https://app.codecov.io/github/talgalili/gplots?branch=master)
[![R-CMD-check](https://github.com/talgalili/gplots/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/talgalili/gplots/actions/workflows/R-CMD-check.yaml)

**[gplots website](https://talgalili.github.io/gplots/) (built using [pkgdown](https://pkgdown.r-lib.org/))**

# gplots


## Introduction

This repo is to keep the gplots package alive. I don't plan to develop new features, but if you'll send pull requests I'm willing to review them.


### Latest news

You can see the most recent changes to the package in the [NEWS.md file](https://talgalili.github.io/gplots/news/index.html)



### Code of conduct

Please note that this project is released with a [Contributor Code of Conduct](https://github.com/talgalili/gplots/blob/master/CODE_OF_CONDUCT.md). By participating in this project you agree to abide by its terms.



## Installation

To install the stable version on CRAN:

```r
install.packages('gplots')
```


And then you may load the package using:

```R
library("gplots")
```

## Usage

TODO


## Contact

You are welcome to:

* Ask questions on: <https://stackoverflow.com/questions/tagged/gplots>
* Submit suggestions and bug-reports at: <https://github.com/talgalili/gplots/issues>
* Send a pull request on: <https://github.com/talgalili/gplots/>
* Compose a friendly e-mail to: <tal.galili@gmail.com>

